@@ -1,9 +1,8 @@
 /*
    Dialog box features module for the Midnight Commander
 
-   Copyright (C) 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2007, 2009, 2010, 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2014
+   Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
 
@@ -34,7 +33,6 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>              /* open() */
 
 #include "lib/global.h"
 
@@ -42,15 +40,18 @@
 #include "lib/skin.h"
 #include "lib/tty/key.h"
 #include "lib/strutil.h"
-#include "lib/widget.h"
 #include "lib/fileloc.h"        /* MC_HISTORY_FILE */
 #include "lib/event.h"          /* mc_event_raise() */
 
+#include "lib/widget.h"
+#include "lib/widget/mouse.h"
+
 /*** global variables ****************************************************************************/
 
 /* Color styles for normal and error dialogs */
 dlg_colors_t dialog_colors;
 dlg_colors_t alarm_colors;
+dlg_colors_t listbox_colors;
 
 /* Primitive way to check if the the current dialog is our dialog */
 /* This is needed by async routines like load_prompt */
@@ -200,8 +201,8 @@ dlg_unfocus (WDialog * h)
 static int
 dlg_find_widget_callback (const void *a, const void *b)
 {
-    const Widget *w = WIDGET (a);
-    widget_cb_fn f = (widget_cb_fn) b;
+    const Widget *w = CONST_WIDGET (a);
+    const widget_cb_fn f = b;
 
     return (w->callback == f) ? 0 : 1;
 }
@@ -242,6 +243,8 @@ do_select_widget (WDialog * h, GList * w, select_dir_t dir)
         case SELECT_PREV:
             h->current = dlg_widget_prev (h, h->current);
             break;
+        default:
+            break;
         }
     }
     while (h->current != w /* && (WIDGET (h->current->data)->options & W_DISABLED) == 0 */ );
@@ -271,7 +274,7 @@ refresh_cmd (void)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-dlg_execute_cmd (WDialog * h, unsigned long command)
+dlg_execute_cmd (WDialog * h, long command)
 {
     cb_ret_t ret = MSG_HANDLED;
     switch (command)
@@ -340,7 +343,7 @@ dlg_execute_cmd (WDialog * h, unsigned long command)
 static cb_ret_t
 dlg_handle_key (WDialog * h, int d_key)
 {
-    unsigned long command;
+    long command;
 
     command = keybind_lookup_keymap_command (dialog_map, d_key);
 
@@ -354,6 +357,21 @@ dlg_handle_key (WDialog * h, int d_key)
     return MSG_NOT_HANDLED;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * This is the low-level mouse handler.
+ * It receives a Gpm_Event event and translates it into a higher level protocol.
+ */
+static int
+dlg_mouse_translator (Gpm_Event * event, Widget * w)
+{
+    mouse_event_t me;
+
+    me = mouse_translate_event (w, event);
+
+    return mouse_process_event (w, &me);
+}
+
 /* --------------------------------------------------------------------------------------------- */
 
 static int
@@ -361,7 +379,7 @@ dlg_mouse_event (WDialog * h, Gpm_Event * event)
 {
     Widget *wh = WIDGET (h);
 
-    GList *p, *first;
+    GList *p;
 
     /* close the dialog by mouse left click out of dialog area */
     if (mouse_close_dialog && !h->fullscreen && ((event->buttons & GPM_B_LEFT) != 0)
@@ -372,35 +390,34 @@ dlg_mouse_event (WDialog * h, Gpm_Event * event)
         return MOU_NORMAL;
     }
 
-    if (wh->mouse != NULL)
+    if (wh->mouse_callback != NULL)
     {
         int mou;
 
-        mou = wh->mouse (event, wh);
+        mou = dlg_mouse_translator (event, wh);
         if (mou != MOU_UNHANDLED)
             return mou;
     }
 
-    first = h->current;
-    p = first;
-
+    /* send the event to widgets in reverse Z-order */
+    p = g_list_last (h->widgets);
     do
     {
         Widget *w = WIDGET (p->data);
 
-        p = dlg_widget_prev (h, p);
-
-        if ((w->options & W_DISABLED) == 0 && w->mouse != NULL)
+        if ((w->options & W_DISABLED) == 0 && w->mouse_callback != NULL)
         {
             /* put global cursor position to the widget */
             int ret;
 
-            ret = w->mouse (event, w);
+            ret = dlg_mouse_translator (event, w);
             if (ret != MOU_UNHANDLED)
                 return ret;
         }
+
+        p = g_list_previous (p);
     }
-    while (p != first);
+    while (p != NULL);
 
     return MOU_UNHANDLED;
 }
@@ -578,12 +595,37 @@ frontend_dlg_run (WDialog * h)
 static int
 dlg_find_widget_by_id (gconstpointer a, gconstpointer b)
 {
-    Widget *w = WIDGET (a);
+    const Widget *w = CONST_WIDGET (a);
     unsigned long id = GPOINTER_TO_UINT (b);
 
     return w->id == id ? 0 : 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+dlg_set_top_or_bottom_widget (void *w, gboolean set_top)
+{
+    Widget *widget = WIDGET (w);
+    WDialog *h = widget->owner;
+    GList *l;
+
+    l = g_list_find (h->widgets, w);
+    if (l == NULL)
+        abort ();               /* widget is not in dialog, this should not happen */
+
+    /* unfocus prevoius widget and focus current one before widget reordering */
+    if (set_top && h->state == DLG_ACTIVE)
+        do_select_widget (h, l, SELECT_EXACT);
+
+    /* widget reordering */
+    h->widgets = g_list_remove_link (h->widgets, l);
+    if (set_top)
+        h->widgets = g_list_concat (h->widgets, l);
+    else
+        h->widgets = g_list_concat (l, h->widgets);
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -700,7 +742,7 @@ dlg_set_position (WDialog * h, int y1, int x1, int y2, int x2)
 }
 
 /* --------------------------------------------------------------------------------------------- */
-/** this function sets only size, leaving positioning to automatic methods */
+/** Set dialog size and position */
 
 void
 dlg_set_size (WDialog * h, int lines, int cols)
@@ -708,14 +750,19 @@ dlg_set_size (WDialog * h, int lines, int cols)
     int x = WIDGET (h)->x;
     int y = WIDGET (h)->y;
 
-    if (h->flags & DLG_CENTER)
+    if ((h->flags & DLG_CENTER) != 0)
     {
         y = (LINES - lines) / 2;
         x = (COLS - cols) / 2;
     }
 
-    if ((h->flags & DLG_TRYUP) && (y > 3))
-        y -= 2;
+    if ((h->flags & DLG_TRYUP) != 0)
+    {
+        if (y > 3)
+            y -= 2;
+        else if (y == 3)
+            y = 2;
+    }
 
     dlg_set_position (h, y, x, y + lines, x + cols);
 }
@@ -766,7 +813,7 @@ dlg_default_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, v
 
 WDialog *
 dlg_create (gboolean modal, int y1, int x1, int lines, int cols,
-            const int *colors, widget_cb_fn callback, mouse_h mouse_handler,
+            const int *colors, widget_cb_fn callback, widget_mouse_cb_fn mouse_callback,
             const char *help_ctx, const char *title, dlg_flags_t flags)
 {
     WDialog *new_d;
@@ -775,13 +822,12 @@ dlg_create (gboolean modal, int y1, int x1, int lines, int cols,
     new_d = g_new0 (WDialog, 1);
     w = WIDGET (new_d);
     widget_init (w, y1, x1, lines, cols, (callback != NULL) ? callback : dlg_default_callback,
-                 mouse_handler);
+                 mouse_callback);
     widget_want_cursor (w, FALSE);
 
     new_d->state = DLG_CONSTRUCT;
     new_d->modal = modal;
-    if (colors != NULL)
-        memmove (new_d->color, colors, sizeof (dlg_colors_t));
+    new_d->color = colors;
     new_d->help_ctx = help_ctx;
     new_d->flags = flags;
     new_d->data = NULL;
@@ -824,6 +870,12 @@ dlg_set_default_colors (void)
     alarm_colors[DLG_COLOR_HOT_NORMAL] = ERROR_HOT_NORMAL;
     alarm_colors[DLG_COLOR_HOT_FOCUS] = ERROR_HOT_FOCUS;
     alarm_colors[DLG_COLOR_TITLE] = ERROR_TITLE;
+
+    listbox_colors[DLG_COLOR_NORMAL] = PMENU_ENTRY_COLOR;
+    listbox_colors[DLG_COLOR_FOCUS] = PMENU_SELECTED_COLOR;
+    listbox_colors[DLG_COLOR_HOT_NORMAL] = PMENU_ENTRY_COLOR;
+    listbox_colors[DLG_COLOR_HOT_FOCUS] = PMENU_SELECTED_COLOR;
+    listbox_colors[DLG_COLOR_TITLE] = PMENU_TITLE_COLOR;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1015,7 +1067,7 @@ find_widget_type (const WDialog * h, widget_cb_fn callback)
 {
     GList *w;
 
-    w = g_list_find_custom (h->widgets, callback, dlg_find_widget_callback);
+    w = g_list_find_custom (h->widgets, (gconstpointer) callback, dlg_find_widget_callback);
 
     return (w == NULL) ? NULL : WIDGET (w->data);
 }
@@ -1060,7 +1112,6 @@ dlg_select_widget (void *w)
 }
 
 /* --------------------------------------------------------------------------------------------- */
-
 /**
  * Set widget at top of widget list and make it current.
  */
@@ -1068,22 +1119,18 @@ dlg_select_widget (void *w)
 void
 dlg_set_top_widget (void *w)
 {
-    Widget *widget = WIDGET (w);
-    WDialog *h = widget->owner;
-    GList *l;
+    dlg_set_top_or_bottom_widget (w, TRUE);
+}
 
-    l = g_list_find (h->widgets, w);
-    if (l == NULL)
-        abort ();               /* widget is not in dialog, this should not happen */
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Set widget at bottom of widget list.
+ */
 
-    /* unfocus prevoius widget and focus current one before widget reordering */
-    if (h->state == DLG_ACTIVE)
-        do_select_widget (h, l, SELECT_EXACT);
-
-    /* widget reordering */
-    h->widgets = g_list_remove_link (h->widgets, l);
-    h->widgets = g_list_concat (h->widgets, l);
-    h->current = l;
+void
+dlg_set_bottom_widget (void *w)
+{
+    dlg_set_top_or_bottom_widget (w, FALSE);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1215,11 +1262,8 @@ dlg_process_event (WDialog * h, int key, Gpm_Event * event)
         if (tty_got_interrupt ())
             if (send_message (h, NULL, MSG_ACTION, CK_Cancel, NULL) != MSG_HANDLED)
                 dlg_execute_cmd (h, CK_Cancel);
-
-        return;
     }
-
-    if (key == EV_MOUSE)
+    else if (key == EV_MOUSE)
         h->mouse_status = dlg_mouse_event (h, event);
     else
         dlg_key_event (h, key);
