@@ -1,7 +1,7 @@
 /*
    Some misc dialog boxes for the program.
 
-   Copyright (C) 1994-2015
+   Copyright (C) 1994-2016
    Free Software Foundation, Inc.
 
    Written by:
@@ -101,7 +101,6 @@ static const int panel_listing_brief_idx = 1;
 static const int panel_listing_user_idx = 3;
 
 static char **status_format;
-static int listing_user_hotkey = 'u';
 static unsigned long panel_listing_types_id, panel_user_format_id, panel_brief_cols_id;
 static unsigned long mini_user_status_id, mini_user_format_id;
 
@@ -129,7 +128,7 @@ configure_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, voi
 {
     switch (msg)
     {
-    case MSG_ACTION:
+    case MSG_NOTIFY:
         /* message from "Single press" checkbutton */
         if (sender != NULL && sender->id == configure_old_esc_mode_id)
         {
@@ -251,62 +250,7 @@ panel_listing_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm,
 
     switch (msg)
     {
-    case MSG_KEY:
-        if (parm == '\n')
-        {
-            Widget *wi;
-
-            wi = dlg_find_by_id (h, panel_listing_types_id);
-            if (widget_is_active (wi))
-            {
-                WInput *in;
-
-                in = INPUT (dlg_find_by_id (h, mini_user_format_id));
-                input_assign_text (in, status_format[RADIO (wi)->sel]);
-                dlg_stop (h);
-                return MSG_HANDLED;
-            }
-
-            wi = dlg_find_by_id (h, panel_user_format_id);
-            if (widget_is_active (wi))
-            {
-                h->ret_value = B_USER + 6;
-                dlg_stop (h);
-                return MSG_HANDLED;
-            }
-
-            wi = dlg_find_by_id (h, mini_user_format_id);
-            if (widget_is_active (wi))
-            {
-                h->ret_value = B_USER + 7;
-                dlg_stop (h);
-                return MSG_HANDLED;
-            }
-        }
-
-        if (g_ascii_tolower (parm) == listing_user_hotkey)
-        {
-            Widget *wi;
-
-            wi = dlg_find_by_id (h, panel_user_format_id);
-            if (widget_is_active (wi))
-            {
-                wi = dlg_find_by_id (h, mini_user_format_id);
-                if (widget_is_active (wi))
-                {
-                    WRadio *r;
-
-                    r = RADIO (dlg_find_by_id (h, panel_listing_types_id));
-                    r->pos = r->sel = panel_listing_user_idx;
-                    dlg_select_widget (WIDGET (r));     /* force redraw */
-                    send_message (h, r, MSG_ACTION, 0, NULL);
-                    return MSG_HANDLED;
-                }
-            }
-        }
-        return MSG_NOT_HANDLED;
-
-    case MSG_ACTION:
+    case MSG_NOTIFY:
         if (sender != NULL && sender->id == panel_listing_types_id)
         {
             WCheck *ch;
@@ -427,7 +371,7 @@ confvfs_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
 {
     switch (msg)
     {
-    case MSG_ACTION:
+    case MSG_NOTIFY:
         /* message from "Always use ftp proxy" checkbutton */
         if (sender != NULL && sender->id == ftpfs_always_use_proxy_id)
         {
@@ -641,7 +585,8 @@ appearance_box (void)
         };
 
         if (quick_dialog (&qdlg) == B_ENTER)
-            mc_config_set_string (mc_main_config, CONFIG_APP_SECTION, "skin", current_skin_name);
+            mc_config_set_string (mc_global.main_config, CONFIG_APP_SECTION, "skin",
+                                  current_skin_name);
         else
             skin_apply (NULL);
     }
@@ -658,7 +603,7 @@ panel_options_box (void)
 {
     int simple_swap;
 
-    simple_swap = mc_config_get_bool (mc_main_config, CONFIG_PANELS_SECTION,
+    simple_swap = mc_config_get_bool (mc_global.main_config, CONFIG_PANELS_SECTION,
                                       "simple_swap", FALSE) ? 1 : 0;
     {
         const char *qsearch_options[] = {
@@ -685,12 +630,14 @@ panel_options_box (void)
                                     NULL),
                     QUICK_SEPARATOR (FALSE),
                     QUICK_SEPARATOR (FALSE),
+                    QUICK_SEPARATOR (FALSE),
                 QUICK_STOP_GROUPBOX,
             QUICK_NEXT_COLUMN,
                 QUICK_START_GROUPBOX (N_("Navigation")),
                     QUICK_CHECKBOX (N_("L&ynx-like motion"), &panels_options.navigate_with_arrows,
                                     NULL),
                     QUICK_CHECKBOX (N_("Pa&ge scrolling"), &panels_options.scroll_pages, NULL),
+                    QUICK_CHECKBOX (N_("Center &scrolling"), &panels_options.scroll_center, NULL),
                     QUICK_CHECKBOX (N_("&Mouse page scrolling"), &panels_options.mouse_move_pages,
                                     NULL),
                 QUICK_STOP_GROUPBOX,
@@ -718,7 +665,7 @@ panel_options_box (void)
             return;
     }
 
-    mc_config_set_bool (mc_main_config, CONFIG_PANELS_SECTION,
+    mc_config_set_bool (mc_global.main_config, CONFIG_PANELS_SECTION,
                         "simple_swap", (gboolean) (simple_swap & C_BOOL));
 
     if (!panels_options.fast_reload_msg_shown && panels_options.fast_reload)
@@ -756,7 +703,7 @@ panel_listing_box (WPanel * panel, int num, char **userp, char **minip, int *use
         for (i = 0; i < LIST_TYPES; i++)
             panel->user_status_format[i] = g_strdup (DEFAULT_USER_FORMAT);
         section = g_strconcat ("Temporal:", p, (char *) NULL);
-        if (!mc_config_has_group (mc_main_config, section))
+        if (!mc_config_has_group (mc_global.main_config, section))
         {
             g_free (section);
             section = g_strdup (p);
@@ -771,7 +718,6 @@ panel_listing_box (WPanel * panel, int num, char **userp, char **minip, int *use
         char *panel_brief_cols_out = NULL;
         char *panel_user_format = NULL;
         char *mini_user_format = NULL;
-        const char *cp;
 
         /* Controls whether the array strings have been translated */
         const char *list_types[LIST_TYPES] = {
@@ -808,11 +754,6 @@ panel_listing_box (WPanel * panel, int num, char **userp, char **minip, int *use
             quick_widgets, panel_listing_callback, NULL
         };
 
-        /* get hotkey of user-defined format string */
-        cp = strchr (_(list_types[panel_listing_user_idx]), '&');
-        if (cp != NULL && *++cp != '\0')
-            listing_user_hotkey = g_ascii_tolower (*cp);
-
         mini_user_status = panel->user_mini_status;
         result = panel->list_type;
         status_format = panel->user_status_format;
@@ -867,8 +808,9 @@ panel_listing_box (WPanel * panel, int num, char **userp, char **minip, int *use
 const panel_field_t *
 sort_box (dir_sort_options_t * op, const panel_field_t * sort_field)
 {
-    const char **sort_orders_names;
-    gsize sort_names_num, i;
+    char **sort_orders_names;
+    gsize i;
+    gsize sort_names_num = 0;
     int sort_idx = 0;
     const panel_field_t *result = NULL;
 
@@ -885,7 +827,7 @@ sort_box (dir_sort_options_t * op, const panel_field_t * sort_field)
         quick_widget_t quick_widgets[] = {
             /* *INDENT-OFF* */
             QUICK_START_COLUMNS,
-                QUICK_RADIO (sort_names_num, sort_orders_names, &sort_idx, NULL),
+                QUICK_RADIO (sort_names_num, (const char **) sort_orders_names, &sort_idx, NULL),
             QUICK_NEXT_COLUMN,
                 QUICK_CHECKBOX (N_("Executable &first"), &op->exec_first, NULL),
                 QUICK_CHECKBOX (N_("Cas&e sensitive"), &op->case_sensitive, NULL),
@@ -909,7 +851,7 @@ sort_box (dir_sort_options_t * op, const panel_field_t * sort_field)
             result = sort_field;
     }
 
-    g_strfreev ((gchar **) sort_orders_names);
+    g_strfreev (sort_orders_names);
 
     return result;
 }
@@ -1094,6 +1036,7 @@ tree_box (const char *current_dir)
     if (dlg_run (dlg) == B_ENTER)
     {
         const vfs_path_t *selected_name;
+
         selected_name = tree_selected_name (mytree);
         val = g_strdup (vfs_path_as_str (selected_name));
     }
@@ -1199,7 +1142,7 @@ configure_vfs (void)
 char *
 cd_dialog (void)
 {
-    const Widget *w = WIDGET (current_panel);
+    const Widget *w = CONST_WIDGET (current_panel);
     char *my_str;
 
     quick_widget_t quick_widgets[] = {
@@ -1295,7 +1238,7 @@ jobs_cmd (void)
     }
 
     x += (int) n_but - 1;
-    cols = max (cols, x + 6);
+    cols = MAX (cols, x + 6);
 
     jobs_dlg = dlg_create (TRUE, 0, 0, lines, cols, dialog_colors, NULL, NULL,
                            "[Background jobs]", _("Background jobs"), DLG_CENTER);
